@@ -7,6 +7,9 @@ use crate::SymbolSize;
 
 use alloc::{vec, vec::Vec};
 
+#[cfg(test)]
+use pretty_assertions::assert_eq;
+
 /// Decode the Reed-Solomon code using a syndrome based decoder.
 ///
 /// See the [module documentation](crate::errorcode) for some implementation details.
@@ -397,7 +400,7 @@ fn find_inv_error_locations_lu(syndomes: &[GF]) -> Result<Vec<GF>, ErrorDecoding
     // step 1: find the error locator polynomial
 
     // build syndrome matrix
-    let mut matrix = vec![GF(0); v * v]; // row major order
+    let mut matrix = vec![GF(0); v * v]; // row-major order
     for i in 0..v {
         for j in 0..v {
             matrix[i * v + j] = syndomes[i + j];
@@ -475,7 +478,7 @@ fn solve_vandermonde_diag() {
         *tmp = GF(1) / *tmp;
     }
     let mut rhs = [GF(66), GF(27), GF(189), GF(255), GF(206)];
-    let mut y1 = rhs.clone();
+    let mut y1 = rhs;
     find_error_values_bp(&mut x[..], &[], &mut y1[..]);
 
     let mut mat = [
