@@ -4,40 +4,45 @@
 //! to iterate over the bit
 //! positions of each codeword in the final symbol, i.e., how the black squares are
 //! mapped to the encoded data as bytes. This is used to write
-//! the encoded into a bitmap, and also to read it from a bitmap.
+//! the encoded data into a bitmap, and also to read it from a bitmap.
 //!
 //! An abstract bitmap struct [Bitmap] is the final output of encoding and the input
 //! for decoding. It also contains helpers for rendering.
 use alloc::{string::String, vec, vec::Vec};
 
-use crate::symbol_size::SymbolSize;
+use crate::symbol_size::{SymbolList, SymbolSize};
+
+#[cfg(test)]
+use pretty_assertions::assert_eq;
 
 mod path;
 
 pub use path::PathSegment;
 
-/// Trait for a visitor to the symbol's bits.
-///
-/// A bit is called "module" in the specification. Each codeword consists
-/// of eight bits (modules).
-///
-/// During traversal the visitor is called with pointers to the
-/// codewords' bits. It can either read of write them.
-pub trait Visitor<B: Bit> {
-    /// Visit the next codewords' bits.
-    fn visit(&mut self, codeword_index: usize, bits: [&mut B; 8]);
+#[derive(Debug, Clone, PartialEq, Eq)]
+pub enum BitmapConversionError {
+    /// The alignment pattern is not correct.
+    Alignment,
+    /// The padding pattern is not correct.
+    Padding,
+    /// The width was zero.
+    ZeroWidth,
+    /// The provided data does not fit the given width.
+    DataSize,
+    /// No symbol size was found matching the data size.
+    SymbolSize,
 }
 
 /// Abstract "bit" type used in [MatrixMap].
-pub trait Bit: Clone + PartialEq + core::fmt::Debug {
+pub trait Bit: Clone + Copy + PartialEq + core::fmt::Debug {
     const LOW: Self;
     const HIGH: Self;
 }
 
 /// Representation of the bits in a Data Matrix symbol without alignment patterns.
+#[derive(Debug, Clone, PartialEq, Eq)]
 pub struct MatrixMap<B: Bit> {
     entries: Vec<B>,
-    visited: Vec<bool>,
     width: usize,
     height: usize,
     extra_vertical_alignments: usize,
@@ -48,38 +53,116 @@ pub struct MatrixMap<B: Bit> {
 impl<M: Bit> MatrixMap<M> {
     /// Create a new, empty matrix for the given symbol size.
     pub fn new(size: SymbolSize) -> Self {
-        let num_data = size.num_data_codewords();
         let setup = size.block_setup();
-        let num_error = setup.num_error_codes();
-
-        let has_padding = size.has_padding_modules();
-        let padding = if has_padding { 4 } else { 0 };
-        let len = (num_data + num_error) * 8 + padding;
-        let mut entries = Vec::with_capacity(len);
-        entries.resize(len, M::LOW);
-
-        let w = setup.width - 2 - setup.extra_vertical_alignments * 2;
-        let h = setup.height - 2 - setup.extra_horizontal_alignments * 2;
-        debug_assert_eq!(w * h, len);
-
+        let w = setup.content_width();
+        let h = setup.content_height();
         Self {
-            entries,
-            visited: vec![],
+            entries: vec![M::LOW; w * h],
             width: w,
             height: h,
             extra_vertical_alignments: setup.extra_vertical_alignments,
             extra_horizontal_alignments: setup.extra_horizontal_alignments,
-            has_padding,
+            has_padding: size.has_padding_modules(),
         }
     }
 
-    // Write a 4x4 padding pattern in the lower right corner if needed.
-    fn write_padding(&mut self) {
-        if !self.has_padding {
-            return;
+    /// Read the data from a bitmap.
+    ///
+    /// The argument `bits` shall reprersent a rectangular image, enumerated starting
+    /// from the top left corner in row-major order. The alignment patterns must be included.
+    pub fn try_from_bits(
+        bits: &[M],
+        width: usize,
+    ) -> Result<(Self, SymbolSize), BitmapConversionError>
+    where
+        M: PartialEq,
+    {
+        if width == 0 {
+            return Err(BitmapConversionError::ZeroWidth);
+        }
+        if bits.len() % width != 0 {
+            return Err(BitmapConversionError::DataSize);
+        }
+        let height = bits.len() / width;
+        let size = SymbolList::all()
+            .iter()
+            .find(|s| {
+                let bs = s.block_setup();
+                bs.width == width && bs.height == height
+            })
+            .ok_or(BitmapConversionError::SymbolSize)?;
+        let setup = size.block_setup();
+        let w = setup.content_width();
+        let h = setup.content_height();
+        let mut entries = Vec::with_capacity(w * h);
+
+        let blk_h = h / (setup.extra_horizontal_alignments + 1);
+        let blk_w = w / (setup.extra_vertical_alignments + 1);
+
+        for row_chunk in bits.chunks((blk_h + 2) * width) {
+            debug_assert_eq!(row_chunk.len(), (blk_h + 2) * width);
+
+            // first row must be alternating, the one before all HIGH
+            let first_row = &row_chunk[..width];
+            let last_row = &row_chunk[(blk_h + 1) * width..];
+            debug_assert_eq!(last_row.len(), width);
+            let alignment_ok = last_row.iter().all(|b| *b == M::HIGH)
+                && first_row
+                    .iter()
+                    .zip([M::HIGH, M::LOW].into_iter().cycle())
+                    .all(|(a, b)| *a == b);
+            if !alignment_ok {
+                return Err(BitmapConversionError::Alignment);
+            }
+
+            let rows = &row_chunk[width..(blk_h + 1) * width];
+            debug_assert_eq!(rows.len(), blk_h * width);
+            debug_assert_eq!(width % (blk_w + 2), 0);
+            let mut alignment_bit = M::LOW;
+            for (j, row) in rows.chunks(blk_w + 2).enumerate() {
+                debug_assert_eq!(row.len(), blk_w + 2);
+                if j % (setup.extra_vertical_alignments + 1) == 0 {
+                    alignment_bit = if alignment_bit == M::LOW {
+                        M::HIGH
+                    } else {
+                        M::LOW
+                    };
+                }
+                let alignment_ok = row[0] == M::HIGH && row[blk_w + 1] == alignment_bit;
+                if !alignment_ok {
+                    return Err(BitmapConversionError::Alignment);
+                }
+                entries.extend_from_slice(&row[1..blk_w + 1]);
+                debug_assert_eq!(row[1..blk_w + 1].len(), blk_w);
+            }
+        }
+        debug_assert_eq!(entries.len(), w * h);
+
+        if size.has_padding_modules() {
+            let padding_ok = entries[entries.len() - 2..] == [M::LOW, M::HIGH]
+                && entries[entries.len() - w - 2..entries.len() - w] == [M::HIGH, M::LOW];
+            if !padding_ok {
+                return Err(BitmapConversionError::Padding);
+            }
+        }
+
+        let matrix_map = Self {
+            entries,
+            width: w,
+            height: h,
+            extra_vertical_alignments: setup.extra_vertical_alignments,
+            extra_horizontal_alignments: setup.extra_horizontal_alignments,
+            has_padding: size.has_padding_modules(),
+        };
+        Ok((matrix_map, size))
+    }
+
+    /// Write a 4x4 padding pattern in the lower right corner if needed.
+    pub fn write_padding(&mut self) {
+        if self.has_padding {
+            *self.bit_mut(self.height - 2, self.width - 2) = M::HIGH;
+            *self.bit_mut(self.height - 1, self.width - 1) = M::HIGH;
         }
-        *self.bit_mut(self.height - 2, self.width - 2) = M::HIGH;
-        *self.bit_mut(self.height - 1, self.width - 1) = M::HIGH;
     }
 
     /// Get the content of the matrix as a bitmap with alignment patterns added.
@@ -91,9 +174,9 @@ impl<M: Bit> MatrixMap<M> {
         let idx = |i: usize, j: usize| i * w + j;
 
         // draw horizontal alignments
-        let xtr_hor = self.extra_horizontal_alignments;
-        let blk_h = (h - 2 * (xtr_hor + 1)) / (xtr_hor + 1);
-        for i in 0..xtr_hor {
+        let extra_hor = self.extra_horizontal_alignments;
+        let blk_h = (h - 2 * (extra_hor + 1)) / (extra_hor + 1);
+        for i in 0..extra_hor {
             let rows_before = 1 + (blk_h + 2) * i + blk_h;
             for j in 0..w {
                 bits[idx(rows_before, j)] = M::HIGH;
@@ -104,9 +187,9 @@ impl<M: Bit> MatrixMap<M> {
         }
 
         // draw vertical alignments
-        let xtr_ver = self.extra_vertical_alignments;
-        let blk_w = (w - 2 * (xtr_ver + 1)) / (xtr_ver + 1);
-        for j in 0..xtr_ver {
+        let extra_ver = self.extra_vertical_alignments;
+        let blk_w = (w - 2 * (extra_ver + 1)) / (extra_ver + 1);
+        for j in 0..extra_ver {
             let cols_before = 1 + (blk_w + 2) * j + blk_w;
             for i in 1..h {
                 bits[idx(i, cols_before + 1)] = M::HIGH;
@@ -139,167 +222,54 @@ impl<M: Bit> MatrixMap<M> {
             i += 1 + (i / blk_h) * 2;
             let mut j = b_i % self.width;
             j += 1 + (j / blk_w) * 2;
-            bits[idx(i, j)] = b.clone();
+            bits[idx(i, j)] = *b;
         }
 
         Bitmap { width: w, bits }
     }
 
-    /// Traverse the symbol in codeword order and call the visitor.
-    pub fn traverse<V: Visitor<M>>(&mut self, visitor: &mut V) {
-        let nrow = self.height as i16;
-        let ncol = self.width as i16;
-        self.visited = vec![false; (nrow * ncol) as usize];
-
-        // starting in the correct location for first character, bit 8
-        let mut i = 4;
-        let mut j = 0;
-        let mut codeword_idx = 0;
-
-        loop {
-            // repeatedly first check for one of the special corner cases
-            if i == nrow && j == 0 {
-                visitor.visit(codeword_idx, self.corner1());
-                codeword_idx += 1;
-            }
-            if i == nrow - 2 && j == 0 && ncol % 4 != 0 {
-                visitor.visit(codeword_idx, self.corner2());
-                codeword_idx += 1;
-            }
-            if i == nrow - 2 && j == 0 && ncol % 8 == 4 {
-                visitor.visit(codeword_idx, self.corner3());
-                codeword_idx += 1;
-            }
-            if i == nrow + 4 && j == 2 && ncol % 8 == 0 {
-                visitor.visit(codeword_idx, self.corner4());
-                codeword_idx += 1;
-            }
-            // sweep upward diagonally
-            loop {
-                if i < nrow && j >= 0 && !self.visited[(i * ncol + j) as usize] {
-                    visitor.visit(codeword_idx, self.utah(i, j));
-                    codeword_idx += 1;
-                }
-                i -= 2;
-                j += 2;
-                if !(i >= 0 && j < ncol) {
-                    break;
-                }
-            }
-            i += 1;
-            j += 3;
-
-            // sweep downard diagonally
-            loop {
-                if i >= 0 && j < ncol && !self.visited[(i * ncol + j) as usize] {
-                    visitor.visit(codeword_idx, self.utah(i, j));
-                    codeword_idx += 1;
-                }
-                i += 2;
-                j -= 2;
-                if !(i < nrow && j >= 0) {
-                    break;
-                }
-            }
-            i += 3;
-            j += 1;
-
-            // until entire map is traversed
-            if !(i < nrow || j < ncol) {
-                break;
-            }
+    /// Traverse the symbol in codeword order and call the function for each position.
+    ///
+    /// The codeword index is given as the first
+    /// argument to `visit`.
+    ///
+    /// The second argument of `visit` contains the bits of the codewords, most significant
+    /// first.
+    pub fn traverse_mut<F>(&mut self, mut visit_fn: F)
+    where
+        F: FnMut(usize, [&mut M; 8]),
+    {
+        IndexTraversal {
+            width: self.width,
+            height: self.height,
         }
-
-        self.write_padding();
+        .run(|idx, indices| {
+            visit_fn(idx, self.bits_mut(indices));
+        });
     }
 
-    // compute idx with wrapping
-    fn idx(&self, mut i: i16, mut j: i16) -> usize {
-        let h = self.height as i16;
-        let w = self.width as i16;
-        if i < 0 {
-            i += h;
-            j += 4 - ((h + 4) % 8);
+    /// Nonmutable version of [traverse_mut](Self::traverse_mut).
+    pub fn traverse<F>(&self, mut visit_fn: F)
+    where
+        F: FnMut(usize, [M; 8]),
+    {
+        IndexTraversal {
+            width: self.width,
+            height: self.height,
         }
-        if j < 0 {
-            j += w;
-            i += 4 - ((w + 4) % 8);
-        }
-        (i * w + j) as usize
-    }
-
-    // compute indices for utah-shaped symbol (the standard symbol)
-    fn utah(&mut self, i: i16, j: i16) -> [&mut M; 8] {
-        self.bits_mut([
-            self.idx(i - 2, j - 2),
-            self.idx(i - 2, j - 1),
-            self.idx(i - 1, j - 2),
-            self.idx(i - 1, j - 1),
-            self.idx(i - 1, j),
-            self.idx(i, j - 2),
-            self.idx(i, j - 1),
-            self.idx(i, j),
-        ])
-    }
-
-    fn corner1(&mut self) -> [&mut M; 8] {
-        let h = self.height as i16;
-        let w = self.width as i16;
-        self.bits_mut([
-            self.idx(h - 1, 0),
-            self.idx(h - 1, 1),
-            self.idx(h - 1, 2),
-            self.idx(0, w - 2),
-            self.idx(0, w - 1),
-            self.idx(1, w - 1),
-            self.idx(2, w - 1),
-            self.idx(3, w - 1),
-        ])
-    }
-
-    fn corner2(&mut self) -> [&mut M; 8] {
-        let h = self.height as i16;
-        let w = self.width as i16;
-        self.bits_mut([
-            self.idx(h - 3, 0),
-            self.idx(h - 2, 0),
-            self.idx(h - 1, 0),
-            self.idx(0, w - 4),
-            self.idx(0, w - 3),
-            self.idx(0, w - 2),
-            self.idx(0, w - 1),
-            self.idx(1, w - 1),
-        ])
-    }
-
-    fn corner3(&mut self) -> [&mut M; 8] {
-        let h = self.height as i16;
-        let w = self.width as i16;
-        self.bits_mut([
-            self.idx(h - 3, 0),
-            self.idx(h - 2, 0),
-            self.idx(h - 1, 0),
-            self.idx(0, w - 2),
-            self.idx(0, w - 1),
-            self.idx(1, w - 1),
-            self.idx(2, w - 1),
-            self.idx(3, w - 1),
-        ])
-    }
-
-    fn corner4(&mut self) -> [&mut M; 8] {
-        let h = self.height as i16;
-        let w = self.width as i16;
-        self.bits_mut([
-            self.idx(h - 1, 0),
-            self.idx(h - 1, w - 1),
-            self.idx(0, w - 3),
-            self.idx(0, w - 2),
-            self.idx(0, w - 1),
-            self.idx(1, w - 3),
-            self.idx(1, w - 2),
-            self.idx(1, w - 1),
-        ])
+        .run(|idx, indices| {
+            let values = [
+                self.entries[indices[0]],
+                self.entries[indices[1]],
+                self.entries[indices[2]],
+                self.entries[indices[3]],
+                self.entries[indices[4]],
+                self.entries[indices[5]],
+                self.entries[indices[6]],
+                self.entries[indices[7]],
+            ];
+            visit_fn(idx, values);
+        });
     }
 
     fn bit_mut(&mut self, i: usize, j: usize) -> &mut M {
@@ -318,7 +288,6 @@ impl<M: Bit> MatrixMap<M> {
             let idx = indices[*perm_idx as usize];
             let (e, new_rest) = rest[(idx - prev)..].split_first_mut().unwrap();
             refs[*perm_idx as usize] = Some(e);
-            self.visited[idx] = true;
             rest = new_rest;
             prev = idx + 1;
         }
@@ -336,6 +305,221 @@ impl<M: Bit> MatrixMap<M> {
     }
 }
 
+struct IndexTraversal {
+    width: usize,
+    height: usize,
+}
+
+impl IndexTraversal {
+    fn run<F>(&self, mut visit_fn: F)
+    where
+        F: FnMut(usize, [usize; 8]),
+    {
+        let nrow = self.height as i16;
+        let ncol = self.width as i16;
+        let mut visited = vec![false; (nrow * ncol) as usize];
+
+        // starting in the correct location for first character, bit 8
+        let mut i = 4;
+        let mut j = 0;
+        let mut codeword_idx = 0;
+
+        macro_rules! visit {
+            ($indices:expr) => {
+                let ii = $indices;
+                for v in ii {
+                    visited[v] = true;
+                }
+                visit_fn(codeword_idx, ii);
+                codeword_idx += 1;
+            };
+        }
+
+        loop {
+            // repeatedly first check for one of the special corner cases
+            if i == nrow && j == 0 {
+                visit!(self.corner1());
+            }
+            if i == nrow - 2 && j == 0 && ncol % 4 != 0 {
+                visit!(self.corner2());
+            }
+            if i == nrow - 2 && j == 0 && ncol % 8 == 4 {
+                visit!(self.corner3());
+            }
+            if i == nrow + 4 && j == 2 && ncol % 8 == 0 {
+                visit!(self.corner4());
+            }
+            // sweep upward diagonally
+            loop {
+                if i < nrow && j >= 0 && !visited[(i * ncol + j) as usize] {
+                    visit!(self.utah(i, j));
+                }
+                i -= 2;
+                j += 2;
+                if !(i >= 0 && j < ncol) {
+                    break;
+                }
+            }
+            i += 1;
+            j += 3;
+
+            // sweep downard diagonally
+            loop {
+                if i >= 0 && j < ncol && !visited[(i * ncol + j) as usize] {
+                    visit!(self.utah(i, j));
+                }
+                i += 2;
+                j -= 2;
+                if !(i < nrow && j >= 0) {
+                    break;
+                }
+            }
+            i += 3;
+            j += 1;
+
+            // until entire map is traversed
+            if !(i < nrow || j < ncol) {
+                break;
+            }
+        }
+    }
+
+    /// Compute index with wrapping
+    fn idx(&self, mut i: i16, mut j: i16) -> usize {
+        let h = self.height as i16;
+        let w = self.width as i16;
+        if i < 0 {
+            i += h;
+            j += 4 - ((h + 4) % 8);
+        }
+        if j < 0 {
+            j += w;
+            i += 4 - ((w + 4) % 8);
+        }
+        // this is needed for DMRE sizes
+        if i >= h {
+            i -= h;
+        }
+        (i * w + j) as usize
+    }
+
+    /// Compute indices for utah-shaped symbol (the standard symbol)
+    fn utah(&self, i: i16, j: i16) -> [usize; 8] {
+        [
+            self.idx(i - 2, j - 2),
+            self.idx(i - 2, j - 1),
+            self.idx(i - 1, j - 2),
+            self.idx(i - 1, j - 1),
+            self.idx(i - 1, j),
+            self.idx(i, j - 2),
+            self.idx(i, j - 1),
+            self.idx(i, j),
+        ]
+    }
+
+    fn corner1(&self) -> [usize; 8] {
+        let h = self.height as i16;
+        let w = self.width as i16;
+        [
+            self.idx(h - 1, 0),
+            self.idx(h - 1, 1),
+            self.idx(h - 1, 2),
+            self.idx(0, w - 2),
+            self.idx(0, w - 1),
+            self.idx(1, w - 1),
+            self.idx(2, w - 1),
+            self.idx(3, w - 1),
+        ]
+    }
+
+    fn corner2(&self) -> [usize; 8] {
+        let h = self.height as i16;
+        let w = self.width as i16;
+        [
+            self.idx(h - 3, 0),
+            self.idx(h - 2, 0),
+            self.idx(h - 1, 0),
+            self.idx(0, w - 4),
+            self.idx(0, w - 3),
+            self.idx(0, w - 2),
+            self.idx(0, w - 1),
+            self.idx(1, w - 1),
+        ]
+    }
+
+    fn corner3(&self) -> [usize; 8] {
+        let h = self.height as i16;
+        let w = self.width as i16;
+        [
+            self.idx(h - 3, 0),
+            self.idx(h - 2, 0),
+            self.idx(h - 1, 0),
+            self.idx(0, w - 2),
+            self.idx(0, w - 1),
+            self.idx(1, w - 1),
+            self.idx(2, w - 1),
+            self.idx(3, w - 1),
+        ]
+    }
+
+    fn corner4(&self) -> [usize; 8] {
+        let h = self.height as i16;
+        let w = self.width as i16;
+        [
+            self.idx(h - 1, 0),
+            self.idx(h - 1, w - 1),
+            self.idx(0, w - 3),
+            self.idx(0, w - 2),
+            self.idx(0, w - 1),
+            self.idx(1, w - 3),
+            self.idx(1, w - 2),
+            self.idx(1, w - 1),
+        ]
+    }
+}
+
+impl MatrixMap<bool> {
+    /// Create a MatrixMap and fills with codewords.
+    pub fn new_with_codewords(data: &[u8], symbol_size: SymbolSize) -> Self {
+        // FIXME: Should not panic if data is too short
+        let mut m = Self::new(symbol_size);
+        m.copy_from_codewords(data);
+        m
+    }
+
+    /// Copy the data from the codewords to the corresponding positions.
+    ///
+    /// Also writes a padding pattern if necessary.
+    ///
+    /// # Panics
+    ///
+    /// Panics if the data is too short.
+    fn copy_from_codewords(&mut self, data: &[u8]) {
+        self.traverse_mut(|idx, bits| {
+            let mut codeword = data[idx];
+            for bit in bits.into_iter().rev() {
+                *bit = codeword & 1 == 1;
+                codeword >>= 1;
+            }
+        });
+        self.write_padding();
+    }
+
+    /// Extract the codewords.
+    ///
+    /// This includes the error correction codewords.
+    pub fn codewords(&self) -> Vec<u8> {
+        let mut data = vec![0; self.entries.len() / 8];
+        self.traverse(|idx, bits| {
+            let codeword = &mut data[idx];
+            for bit in bits {
+                *codeword = (*codeword << 1) | (bit as u8);
+            }
+        });
+        data
+    }
+}
+
 /// An abstract bitmap.
 ///
 /// Contains helpers for rendering the content. For rendering targets which
@@ -352,6 +536,30 @@ impl Bit for bool {
 }
 
 impl<B: Bit> Bitmap<B> {
+    /// Create a new Bitmap.
+    ///
+    /// This allows you to use the rendering helpers also for, say, QR codes:
+    ///
+    /// ```rust
+    /// # use datamatrix::placement::Bitmap;
+    /// use qrcode::{QrCode, Color};
+    ///
+    /// let code = QrCode::new(b"Hello, World!!").unwrap();
+    /// let width = code.width();
+    /// let bits = code.into_colors().into_iter().map(|c| c == Color::Dark);
+    /// let bitmap = Bitmap::new(bits, width);
+    /// print!("{}", bitmap.unicode());
+    /// ```
+    // pub fn new<T: IntoIterator<Item = I>, I: core::borrow::Borrow<B>>(
+    pub fn new<T>(bits: T, width: usize) -> Self
+    where
+        T: IntoIterator<Item = B>,
+    {
+        let bits = Vec::from_iter(bits);
+        assert_eq!(bits.len() % width, 0);
+        Self { bits, width }
+    }
+
     /// Return the width of the bitmap (no quiet zone included).
     pub fn width(&self) -> usize {
         self.width
@@ -370,13 +578,13 @@ impl<B: Bit> Bitmap<B> {
         const BORDER: usize = 1;
         const INVERT: bool = false;
         const CHAR: [char; 4] = [' ', '▄', '▀', '█'];
-        let height = self.bits.len() / self.width;
+        let height = self.height();
         let get = |i: usize, j: usize| -> usize {
             let res =
                 if i < BORDER || i >= BORDER + height || j < BORDER || j >= BORDER + self.width {
                     B::LOW
                 } else if i - BORDER < height && j - BORDER < self.width {
-                    self.bits[(i - BORDER) * self.width + (j - BORDER)].clone()
+                    self.bits[(i - BORDER) * self.width + (j - BORDER)]
                 } else {
                     B::LOW
                 };
@@ -435,6 +643,11 @@ impl<B: Bit> Bitmap<B> {
             .filter(|(_i, b)| **b == B::HIGH)
             .map(move |(i, _b)| (i % w, i / w))
     }
+
+    #[doc(hidden)]
+    pub fn bits(&self) -> &[B] {
+        &self.bits
+    }
 }
 
 #[cfg(test)]
@@ -446,19 +659,14 @@ mod tests {
         const HIGH: Self = (0, 1);
     }
 
-    struct LogVisitor;
-
-    impl super::Visitor<(u16, u8)> for LogVisitor {
-        fn visit(&mut self, cw: usize, bits: [&mut (u16, u8); 8]) {
+    pub fn log(s: super::SymbolSize) -> Vec<(u16, u8)> {
+        let mut m = super::MatrixMap::<(u16, u8)>::new(s);
+        m.traverse_mut(|cw, bits| {
             for i in 0..8 {
                 *bits[i as usize] = ((cw + 1) as u16, (i + 1) as u8);
             }
-        }
-    }
-
-    pub fn log(s: super::SymbolSize) -> Vec<(u16, u8)> {
-        let mut m = super::MatrixMap::<(u16, u8)>::new(s);
-        m.traverse(&mut LogVisitor);
+        });
+        m.write_padding();
         m.entries
     }
 }
@@ -513,3 +721,22 @@ fn test_8x32() {
     ];
     assert_eq!(&log, &should);
 }
+
+#[test]
+fn test_from_bits_all() {
+    let mut random_map = crate::test::random_maps();
+    for size in SymbolList::all() {
+        let map = random_map(size);
+        let bitmap = map.bitmap();
+        let (map2, _size) = MatrixMap::try_from_bits(&bitmap.bits, bitmap.width).unwrap();
+        assert_eq!(map.entries, map2.entries);
+    }
+}
+
+#[test]
+fn test_bitmap_new() {
+    Bitmap::new(vec![true, false], 2);
+    Bitmap::new([true, false], 2);
+    let data = &[true, false];
+    Bitmap::new(data.iter().cloned(), 2);
+}
