@@ -1,14 +1,22 @@
+use core::cmp::{Ordering, PartialOrd};
 use core::fmt::Debug;
+use core::iter::{Extend, FromIterator, IntoIterator};
 use core::ops::RangeBounds;
 
-use arrayvec::ArrayVec;
+use alloc::collections::BTreeSet;
 
 #[cfg(test)]
 use alloc::{vec, vec::Vec};
 
-type SymbolVec = ArrayVec<SymbolSize, 48>;
+#[cfg(test)]
+use enum_iterator::IntoEnumIterator;
 
-#[derive(Debug, PartialEq)]
+#[cfg(test)]
+use pretty_assertions::assert_eq;
+
+type SymbolCollection = BTreeSet<SymbolSize>;
+
+#[derive(Debug, Clone, PartialEq, Eq)]
 /// Set of [symbol sizes](SymbolSize) the encoder is allowed to use.
 ///
 /// Specifies a list of symbol sizes the encoder will pick from. The smallest
@@ -46,7 +54,7 @@ type SymbolVec = ArrayVec<SymbolSize, 48>;
 /// );
 /// ```
 pub struct SymbolList {
-    symbols: SymbolVec,
+    symbols: SymbolCollection,
 }
 
 impl SymbolList {
@@ -57,74 +65,35 @@ impl SymbolList {
     ///
     /// DMRE stands for Data Matrix Rectangular Extensions.
     pub fn with_extended_rectangles() -> Self {
-        let symbols: SymbolVec = SYMBOL_SIZES.iter().cloned().collect();
-        Self::with_whitelist(&symbols)
-    }
-
-    #[deprecated(note = "use with_extended_rectangles()")]
-    #[doc(hidden)]
-    pub fn with_dmre() -> Self {
-        Self::with_extended_rectangles()
+        Self::with_whitelist(SYMBOL_SIZES.iter().cloned())
     }
 
     /// Remove all non-square symbols from the current selection.
     pub fn enforce_square(mut self) -> Self {
-        self.symbols = self.symbols.into_iter().filter(|s| s.is_square()).collect();
+        self.symbols.retain(|s| s.is_square());
         self
     }
 
     /// Remove all square symbols from the current selection.
-    ///
-    /// Although squares rectangles mathematically they are not removed.
     pub fn enforce_rectangular(mut self) -> Self {
-        self.symbols = self
-            .symbols
-            .into_iter()
-            .filter(|s| !s.is_square())
-            .collect();
+        self.symbols.retain(|s| !s.is_square());
         self
     }
 
     /// Only keep symbols with width in the given range.
     pub fn enforce_width_in<R: RangeBounds<usize>>(mut self, bounds: R) -> Self {
-        self.symbols = self
-            .symbols
-            .into_iter()
-            .filter(|s| bounds.contains(&s.block_setup().width))
-            .collect();
+        self.symbols
+            .retain(|s| bounds.contains(&s.block_setup().width));
         self
     }
 
-    #[deprecated(note = "use enforce_width_in")]
-    #[doc(hidden)]
-    pub fn width_range(self, min_width: usize, max_width: usize) -> Self {
-        if min_width <= max_width {
-            self.enforce_width_in(min_width..=max_width)
-        } else {
-            [].into()
-        }
-    }
-
     /// Only keep symbols with height in the given range.
     pub fn enforce_height_in<R: RangeBounds<usize>>(mut self, bounds: R) -> Self {
-        self.symbols = self
-            .symbols
-            .into_iter()
-            .filter(|s| bounds.contains(&s.block_setup().height))
-            .collect();
+        self.symbols
+            .retain(|s| bounds.contains(&s.block_setup().height));
         self
     }
 
-    #[deprecated(note = "use enforce_height_in")]
-    #[doc(hidden)]
-    pub fn height_range(self, min_height: usize, max_height: usize) -> Self {
-        if min_height <= max_height {
-            self.enforce_height_in(min_height..=max_height)
-        } else {
-            [].into()
-        }
-    }
-
     /// Create a symbol list containing only the given symbols.
     ///
     /// The list does not need to be sorted.
@@ -133,16 +102,31 @@ impl SymbolList {
     ///
     /// The call panics if the slice contains more elements than symbol
     /// sizes exist.
-    pub fn with_whitelist(whitelist: &[SymbolSize]) -> Self {
-        let mut symbols: SymbolVec = whitelist.iter().cloned().collect();
-        symbols.sort_by_cached_key(|s| s.num_data_codewords());
-        Self { symbols }
+    pub fn with_whitelist<I>(whitelist: I) -> Self
+    where
+        I: IntoIterator<Item = SymbolSize>,
+    {
+        Self::from_iter(whitelist)
     }
 
-    pub fn iter_symbols(&self) -> impl Iterator<Item = SymbolSize> + '_ {
+    pub fn iter(&self) -> impl Iterator<Item = SymbolSize> + '_ {
         self.symbols.iter().cloned()
     }
 
+    pub fn is_empty(&self) -> bool {
+        self.symbols.is_empty()
+    }
+
+    /// Get a list with all supported symbol sizes.
+    pub fn all() -> Self {
+        Self::with_extended_rectangles()
+    }
+
+    /// Check if a symbol size is in this symbol list.
+    pub fn contains(&self, symbol_size: &SymbolSize) -> bool {
+        self.iter().any(|s| s == *symbol_size)
+    }
+
     pub(crate) fn max_capacity(&self) -> usize {
         self.symbols
             .iter()
@@ -160,7 +144,7 @@ impl SymbolList {
 
     pub(crate) fn upper_limit_for_number_of_codewords(&self, input_len: usize) -> Option<usize> {
         if self.symbols.len() == 1 {
-            Some(self.symbols[0].num_data_codewords())
+            self.symbols.iter().next().map(|s| s.num_data_codewords())
         } else {
             // Min case, try to find a good upper limit
             self.symbols
@@ -175,26 +159,49 @@ impl SymbolList {
     }
 }
 
+impl IntoIterator for SymbolList {
+    type Item = SymbolSize;
+    type IntoIter = <SymbolCollection as IntoIterator>::IntoIter;
+
+    fn into_iter(self) -> Self::IntoIter {
+        self.symbols.into_iter()
+    }
+}
+
+impl FromIterator<SymbolSize> for SymbolList {
+    fn from_iter<T: IntoIterator<Item = SymbolSize>>(iter: T) -> Self {
+        Self {
+            symbols: SymbolCollection::from_iter(iter),
+        }
+    }
+}
+
+impl Extend<SymbolSize> for SymbolList {
+    fn extend<T>(&mut self, iter: T)
+    where
+        T: IntoIterator<Item = SymbolSize>,
+    {
+        self.symbols.extend(iter);
+    }
+}
+
 impl Default for SymbolList {
+    /// Create a symbol list with all but the DMRE symbol sizes.
     fn default() -> Self {
-        let symbols: SymbolVec = SYMBOL_SIZES
-            .iter()
-            .cloned()
-            .filter(|s| !s.is_dmre())
-            .collect();
-        Self::with_whitelist(&symbols)
+        let symbols = SYMBOL_SIZES.iter().cloned().filter(|s| !s.is_dmre());
+        Self::with_whitelist(symbols)
     }
 }
 
 impl From<SymbolSize> for SymbolList {
     fn from(size: SymbolSize) -> SymbolList {
-        SymbolList::with_whitelist(&[size])
+        SymbolList::with_whitelist([size])
     }
 }
 
 impl<const N: usize> From<[SymbolSize; N]> for SymbolList {
     fn from(other: [SymbolSize; N]) -> SymbolList {
-        SymbolList::with_whitelist(&other)
+        SymbolList::with_whitelist(other)
     }
 }
 
@@ -227,8 +234,12 @@ pub(crate) struct BlockSetup {
 }
 
 impl BlockSetup {
-    pub fn num_error_codes(&self) -> usize {
-        self.num_ecc_blocks * self.num_ecc_per_block
+    pub(crate) fn content_width(&self) -> usize {
+        self.width - 2 - self.extra_vertical_alignments * 2
+    }
+
+    pub(crate) fn content_height(&self) -> usize {
+        self.height - 2 - self.extra_horizontal_alignments * 2
     }
 }
 
@@ -237,7 +248,8 @@ impl BlockSetup {
 /// The number behind a variant, e.g., [Square10](SymbolSize::Square10),
 /// describes the number of modules (the tiny black squares) the symbol is
 /// tall/wide.
-#[derive(Debug, Copy, Clone, PartialEq)]
+#[derive(Debug, Copy, Clone, PartialEq, Eq)]
+#[cfg_attr(test, derive(IntoEnumIterator))]
 pub enum SymbolSize {
     Square10,
     Square12,
@@ -310,23 +322,18 @@ pub enum SymbolSize {
 
 #[rustfmt::skip]
 const SYMBOL_SIZES: &[SymbolSize] = &[
-    SymbolSize::Square10, SymbolSize::Square12, SymbolSize::Rect8x18,
-    SymbolSize::Square14, SymbolSize::Rect8x32, SymbolSize::Square16,
-    SymbolSize::Rect12x26, SymbolSize::Square18, SymbolSize::Square20,
-    SymbolSize::Rect12x36, SymbolSize::Square22, SymbolSize::Rect16x36,
-    SymbolSize::Square24, SymbolSize::Square26, SymbolSize::Rect16x48,
-    SymbolSize::Square32, SymbolSize::Square36, SymbolSize::Square40,
-    SymbolSize::Square44, SymbolSize::Square48, SymbolSize::Square52,
-    SymbolSize::Square64, SymbolSize::Square72, SymbolSize::Square80,
-    SymbolSize::Square88, SymbolSize::Square96, SymbolSize::Square104,
-    SymbolSize::Square120, SymbolSize::Square132, SymbolSize::Square144,
-    // DMRE
-    SymbolSize::Rect8x48, SymbolSize::Rect8x64, SymbolSize::Rect8x80,
-    SymbolSize::Rect8x96, SymbolSize::Rect8x120, SymbolSize::Rect8x144,
-    SymbolSize::Rect12x64, SymbolSize::Rect12x88, SymbolSize::Rect16x64,
-    SymbolSize::Rect20x36, SymbolSize::Rect20x44, SymbolSize::Rect20x64,
-    SymbolSize::Rect22x48, SymbolSize::Rect24x48, SymbolSize::Rect24x64,
-    SymbolSize::Rect26x40, SymbolSize::Rect26x48, SymbolSize::Rect26x64,
+    SymbolSize::Square10, SymbolSize::Square12, SymbolSize::Rect8x18, SymbolSize::Square14,
+    SymbolSize::Rect8x32, SymbolSize::Square16, SymbolSize::Rect12x26, SymbolSize::Square18,
+    SymbolSize::Rect8x48, SymbolSize::Square20, SymbolSize::Rect12x36, SymbolSize::Rect8x64,
+    SymbolSize::Square22, SymbolSize::Rect16x36, SymbolSize::Rect8x80, SymbolSize::Square24,
+    SymbolSize::Rect8x96, SymbolSize::Rect12x64, SymbolSize::Square26, SymbolSize::Rect20x36,
+    SymbolSize::Rect16x48, SymbolSize::Rect8x120, SymbolSize::Rect20x44, SymbolSize::Square32,
+    SymbolSize::Rect16x64, SymbolSize::Rect8x144, SymbolSize::Rect12x88, SymbolSize::Rect26x40,
+    SymbolSize::Rect22x48, SymbolSize::Rect24x48, SymbolSize::Rect20x64, SymbolSize::Square36,
+    SymbolSize::Rect26x48, SymbolSize::Rect24x64, SymbolSize::Square40, SymbolSize::Rect26x64,
+    SymbolSize::Square44, SymbolSize::Square48, SymbolSize::Square52, SymbolSize::Square64,
+    SymbolSize::Square72, SymbolSize::Square80, SymbolSize::Square88, SymbolSize::Square96,
+    SymbolSize::Square104, SymbolSize::Square120, SymbolSize::Square132, SymbolSize::Square144,
 ];
 
 impl SymbolSize {
@@ -884,6 +891,14 @@ impl SymbolSize {
         }
     }
 
+    #[cfg(test)]
+    pub(crate) fn num_codewords(&self) -> usize {
+        let num_data = self.num_data_codewords();
+        let setup = self.block_setup();
+        let num_error = setup.num_ecc_blocks * setup.num_ecc_per_block;
+        num_data + num_error
+    }
+
     pub(crate) fn has_padding_modules(&self) -> bool {
         matches!(
             self,
@@ -892,16 +907,62 @@ impl SymbolSize {
     }
 }
 
+impl PartialOrd for SymbolSize {
+    fn partial_cmp(&self, other: &Self) -> Option<Ordering> {
+        Some(self.cmp(other))
+    }
+}
+
+impl Ord for SymbolSize {
+    fn cmp(&self, other: &Self) -> Ordering {
+        fn key(obj: &SymbolSize) -> (usize, usize) {
+            let bs = obj.block_setup();
+            (obj.num_data_codewords(), bs.width.pow(2) + bs.height.pow(2))
+        }
+        key(self).cmp(&key(other))
+    }
+}
+
+#[test]
+fn test_partial_ord_symbol_size() {
+    for a in SYMBOL_SIZES {
+        for b in SYMBOL_SIZES {
+            assert_eq!(
+                a.partial_cmp(b) == Some(core::cmp::Ordering::Equal),
+                a == b,
+                "a = {:?}, b = {:?}",
+                a,
+                b,
+            );
+        }
+    }
+}
+
+#[test]
+fn test_symbol_size_order() {
+    let mut all: Vec<SymbolSize> = SYMBOL_SIZES.into();
+    all.sort_unstable();
+    let all2: Vec<SymbolSize> = SymbolList::all().iter().collect();
+    assert_eq!(&all, &all2,);
+}
+
+#[test]
+fn test_iter_all_symbols() {
+    let mut all: Vec<SymbolSize> = SymbolSize::into_enum_iter().collect();
+    all.sort_unstable();
+    assert_eq!(&all, SYMBOL_SIZES,);
+}
+
 #[test]
 fn test_size_candidates_for_non_auto() {
     let list: SymbolList = SymbolSize::Square10.into();
-    let symbols: Vec<SymbolSize> = list.iter_symbols().collect();
+    let symbols: Vec<SymbolSize> = list.iter().collect();
     assert_eq!(symbols, vec![SymbolSize::Square10]);
 }
 
 #[test]
 fn test_size_candidates_auto() {
-    let all: Vec<SymbolSize> = SymbolList::default().iter_symbols().collect();
+    let all: Vec<SymbolSize> = SymbolList::default().iter().collect();
     let mut expected: Vec<SymbolSize> = SYMBOL_SIZES
         .iter()
         .filter(|s| !s.is_dmre())
@@ -913,10 +974,7 @@ fn test_size_candidates_auto() {
 
 #[test]
 fn test_size_candidates_auto_rect() {
-    let all: Vec<SymbolSize> = SymbolList::default()
-        .enforce_rectangular()
-        .iter_symbols()
-        .collect();
+    let all: Vec<SymbolSize> = SymbolList::default().enforce_rectangular().iter().collect();
     let expected = vec![
         SymbolSize::Rect8x18,
         SymbolSize::Rect8x32,
@@ -930,10 +988,7 @@ fn test_size_candidates_auto_rect() {
 
 #[test]
 fn test_size_candidates_auto_square() {
-    let all: Vec<SymbolSize> = SymbolList::default()
-        .enforce_square()
-        .iter_symbols()
-        .collect();
+    let all: Vec<SymbolSize> = SymbolList::default().enforce_square().iter().collect();
     let expected = vec![
         SymbolSize::Square10,
         SymbolSize::Square12,
@@ -1001,3 +1056,41 @@ fn test_minimal_example_every_symbol() {
         DataMatrix::encode(b"OK", *sym).unwrap();
     }
 }
+
+#[test]
+fn test_distinquishable_by_size() {
+    use alloc::collections::btree_set::BTreeSet;
+    use core::iter::FromIterator;
+
+    let sizes: Vec<_> = SYMBOL_SIZES
+        .iter()
+        .map(|s| {
+            let setup = s.block_setup();
+            (setup.width, setup.height)
+        })
+        .collect();
+    let n = sizes.len();
+    assert_eq!(n, BTreeSet::from_iter(sizes).len());
+}
+
+#[test]
+fn test_list_all() {
+    assert_eq!(SymbolList::all().iter().count(), SYMBOL_SIZES.len());
+
+    for size in SymbolList::all() {
+        assert!(SYMBOL_SIZES.iter().any(|s| *s == size));
+    }
+}
+
+#[test]
+fn test_content_sizes_consistency() {
+    for size in SymbolList::all() {
+        let setup = size.block_setup();
+        let codewords = size.num_codewords();
+        let has_padding = size.has_padding_modules();
+        let padding = if has_padding { 4 } else { 0 };
+        let len = codewords * 8 + padding;
+
+        assert_eq!(len, setup.content_width() * setup.content_height());
+    }
+}
